/*
** SecureX.h : API header file for the SecureX DLL SECUREX.DLL
*/

#ifndef __SECURE_X__
#define __SECURE_X__

/*
** System include files
*/

#include <stddef.h>

/*
** Project specific include files
*/

#include "gen.h"
#include "sxs.h"
#include "index.h"

/*
** Constants
*/

#define DES_BLOCK_SIZE					8				/* standard	DES block encryption size						*/

#define DES_KEY_SIZE					8				/* "														*/
#define AES_KEY_SIZE					32				/* AES key size												*/
#define SHA_KEY_SIZE					64				/* SHA key size												*/

#define DES_ENCRYPT						1				/* perform DES encryption									*/
#define DES_DECRYPT						0				/* perform DES decryption									*/

#define AES_ENCRYPT						1				/* perform AES encryption									*/
#define AES_DECRYPT						0				/* perform AES decryption									*/

#define SHA_CERTIFICATE_SIZE			64				/* SHA certificate size										*/

														/* RSA key generation callback states						*/
#define KEYGEN_GENERATE_PRIMES1			1
#define KEYGEN_GENERATE_PRIMES2			2
#define KEYGEN_TEST_MODULUS				3
#define KEYGEN_CALCULATE_SECRET_KEY		4
#define KEYGEN_GENERATE_MODULUS			5
#define KEYGEN_TEST_KEY					6
#define KEYGEN_SUCCESS					7
#define KEYGEN_FAIL						0
#define KEYGEN_NUM_OF_STATES			7

#define RSAENC_STATE1					1
#define RSAENC_STATE2					2

#define MAX_PASSWORD					255				/* maximum length of a password								*/

														/* Lower and upper bound percentages of password strength. See sxs_cpassword_getstrengthpercent */
const int CPASSWORD_STRENGTH_VERY_WEAK_LOWER = 0;
const int CPASSWORD_STRENGTH_VERY_WEAK_UPPER = 19;
const int CPASSWORD_STRENGTH_WEAK_LOWER = 20;
const int CPASSWORD_STRENGTH_WEAK_UPPER = 39;
const int CPASSWORD_STRENGTH_GOOD_LOWER = 40;
const int CPASSWORD_STRENGTH_GOOD_UPPER = 59;
const int CPASSWORD_STRENGTH_STRONG_LOWER = 60;
const int CPASSWORD_STRENGTH_STRONG_UPPER = 79;
const int CPASSWORD_STRENGTH_VERY_STRONG_LOWER = 80;
const int CPASSWORD_STRENGTH_VERY_STRONG_UPPER = 100;

														/* Password strength classifiers					*/
const int CPASSWORD_STRENGTH_VERY_WEAK = 1;
const int CPASSWORD_STRENGTH_WEAK = 2;
const int CPASSWORD_STRENGTH_GOOD = 3;
const int CPASSWORD_STRENGTH_STRONG = 4;
const int CPASSWORD_STRENGTH_VERY_STRONG = 5;

/*
** Macros
*/

#define DIMOF(a)        (sizeof(a) / sizeof((a)[0]))	/* get dimension of an array						*/
#define OFFSETOF(s, m)  ((size_t)&(((s*)0)->m))			/* get offset of a member within a structure		*/

/*
** Types
*/

typedef void(__stdcall *FNPTR)(int);					/* function pointer type							*/

/*
** Random number generation
*/

extern void __stdcall sxs_randomise(const unsigned int seed);
extern unsigned int __stdcall sxs_urnd(void);
extern unsigned char * __stdcall sxs_urnd_array(unsigned char *s, size_t n);

/*
** DES support functions
*/

extern bool __stdcall sxs_check_parity(unsigned char b);
extern bool __stdcall sxs_check_key_parity_bits(const unsigned char *key);
extern void __stdcall sxs_correct_key_parity_bits(unsigned char *key);
extern bool __stdcall sxs_is_weak_key(const unsigned char key[DES_KEY_SIZE]);

/*
** DES single length
*/

extern void __stdcall sxs_des(unsigned char *ciphertext, unsigned char *plaintext, unsigned char *key, const des_operation_t operation);
extern void __stdcall sxs_des_cbc(unsigned char *ciphertext, unsigned char *plaintext, unsigned char *key, unsigned char *icv, const size_t len, const des_operation_t operation);
extern void __stdcall sxs_genmac(unsigned char *src, const size_t len, unsigned char *icv, unsigned char *kmac);
extern bool __stdcall sxs_vermac(unsigned char *src, const size_t len, unsigned char *icv, unsigned char *kmac, const unsigned char *mac);
extern void __stdcall sxs_owf(unsigned char *dest, unsigned char *src, unsigned char *key, size_t len);

/*
** 2DES
*/

extern void __stdcall sxs_2des(unsigned char *dest, unsigned char *src, unsigned char *icv, unsigned char *key, const des_operation_t operation);
extern void __stdcall sxs_2des_cbc(unsigned char *dest, unsigned char *src, unsigned char *icv, unsigned char *key, const size_t len, const des_operation_t operation);
extern void __stdcall sxs_owf_2des(unsigned char *dest, unsigned char *src, unsigned char *key, unsigned len);

/*
** 3DES (triple-DES)
*/

extern void __stdcall sxs_3des(unsigned char *dest, unsigned char *src, unsigned char *icv, unsigned char *key, const des_operation_t operation);
extern void __stdcall sxs_3des_cbc(unsigned char *dest, unsigned char *src, unsigned char *icv, unsigned char *key, const size_t len, const des_operation_t operation);
extern void __stdcall sxs_owf_3des(unsigned char *dest, unsigned char *src, unsigned char *key, unsigned len);

/*
** RSA
*/

extern int __stdcall sxs_rsaenc(byte Result[], byte Data[], size_t DataLen, byte Exp[], size_t ExpLen, byte Mod[], size_t ModLen, long cb);
extern int __stdcall sxs_testrsakey(unsigned char *mod, int modlen, unsigned char *exp, int explen, long cb);
extern int __stdcall sxs_rsakeygen_progress_states(void);
extern int __stdcall sxs_genrsakey(int ModBitSz, int ModByteSz, unsigned char *modulus, unsigned char *exponent, long cb);

/*
** AES
*/

extern int __stdcall sxs_aes_setkey_enc(aes_context *ctx, const unsigned char *key, unsigned int keysize);
extern int __stdcall sxs_aes_setkey_dec(aes_context *ctx, const unsigned char *key, unsigned int keysize);
extern int __stdcall sxs_aes_crypt_ecb( aes_context *ctx, int mode, const unsigned char input[16], unsigned char output[16]);
extern int __stdcall sxs_aes_crypt_cbc(aes_context *ctx, int mode, size_t length, unsigned char iv[16], const unsigned char *input, unsigned char *output);
extern int __stdcall sxs_aes_crypt_cfb128(aes_context *ctx, int mode, size_t length, size_t *iv_off, unsigned char iv[16], const unsigned char *input, unsigned char *output);
extern int __stdcall sxs_aes_crypt_ctr(aes_context *ctx, size_t length, size_t *nc_off, unsigned char nonce_counter[16], unsigned char stream_block[16], const unsigned char *input, unsigned char *output);
extern int __stdcall sxs_aes_self_test(int verbose);

/*
** SHA4
*/

extern void __stdcall sxs_sha4_starts( sha4_context *ctx, int is384 );
extern void __stdcall sxs_sha4_update( sha4_context *ctx, const unsigned char *input, size_t ilen );
extern void __stdcall sxs_sha4_finish( sha4_context *ctx, unsigned char output[64] );
extern void __stdcall sxs_sha4( const unsigned char *input, size_t ilen, unsigned char output[64], int is384 );
extern int __stdcall sxs_sha4_file( const char *path, unsigned char output[64], int is384 );
extern void __stdcall sxs_sha4_hmac_starts( sha4_context *ctx, const unsigned char *key, size_t keylen, int is384 );
extern void __stdcall sxs_sha4_hmac_update( sha4_context *ctx, const unsigned char *input, size_t ilen );
extern void __stdcall sxs_sha4_hmac_finish( sha4_context *ctx, unsigned char output[SHA_CERTIFICATE_SIZE] );
extern void __stdcall sxs_sha4_hmac_reset( sha4_context *ctx );
extern void __stdcall sxs_sha4_hmac( const unsigned char *key, size_t keylen, const unsigned char *input, size_t ilen, unsigned char output[64], int is384 );
extern int __stdcall sxs_sha4_self_test( int verbose );

/*
** Timer support functions
*/

extern void __stdcall sxs_timer_init(void);
extern void __stdcall sxs_timer_uninit(void);
extern void __stdcall sxs_timer_start(void);
extern unsigned long __stdcall sxs_timer_elapsed_msec(void);
extern void __stdcall sxs_timer_reset(void);
extern unsigned long __stdcall sxs_timer_get_clock(void);

/*
** Password strength calculation routines
*/

extern long __stdcall sxs_cpassword_get_strength_percent(unsigned char bPasswordByteArray[MAX_PASSWORD], size_t sztPasswordLen);
extern long __stdcall sxs_cpassword_classify_strength(int iStrengthPercent);

/*
** Utility and support functions
*/

extern void __stdcall sxs_destroy_sensitive_matter(unsigned char *s, int l);
extern unsigned char __stdcall sxs_lrc(const unsigned char *data, size_t len);
extern void __stdcall sxs_xor(unsigned char result[], unsigned char data1[], unsigned char data2[], size_t len);
extern unsigned char * __stdcall sxs_bintohex(unsigned char *hex, unsigned char *bin, int n);
extern unsigned char * __stdcall sxs_hextobin(unsigned char *bin, unsigned char *hex, int n);
extern void __stdcall sxs_sec_encr(unsigned char key[], unsigned char data[], unsigned char result[]);
extern void __stdcall sxs_sec_decr(unsigned char key[], unsigned char data[], unsigned char result[]);
extern int __stdcall sxs_fib(int x);
extern unsigned char * __stdcall sxs_fib_array(unsigned char *dest, size_t len, int x);

/*
** Indexing utility functions
*/

extern int __stdcall idx_construct(char*, index_loc_t);
extern int __stdcall idx_addkey(char*, char*, long);
extern int __stdcall idx_deletekey(char*, char*);
extern int __stdcall idx_search(char*, char*, long*);
extern int __stdcall idx_diskstat(char*, ext_stat_t*);
extern int __stdcall idx_diskrebuild(char*);
extern int __stdcall idx_disk2mem(char*, char*);
extern int __stdcall idx_mem2disk(char*, char*);
extern int __stdcall idx_destruct(char*);
extern void __stdcall idx_destructall(void);

#endif